@@ -30,20 +30,22 @@
 #include "log_msg.h"
 #include <stdarg.h>
 
+#define ASCII_LEN 16
+
 /* Generic hex dump function.
 */
 void
 hex_dump(const unsigned char *data, const int size)
 {
-    int ln, i, j = 0;
-    char ascii_str[17] = {0};
+    int ln=0, i=0, j=0;
+    char ascii_str[ASCII_LEN+1] = {0};
 
     for(i=0; i<size; i++)
     {
-        if((i % 16) == 0)
+        if((i % ASCII_LEN) == 0)
         {
             printf(" %s\n  0x%.4x:  ", ascii_str, i);
-            memset(ascii_str, 0x0, 17);
+            memset(ascii_str, 0x0, ASCII_LEN-1);
             j = 0;
         }
 
@@ -60,13 +62,14 @@ hex_dump(const unsigned char *data, const int size)
     ln = strlen(ascii_str);
     if(ln > 0)
     {
-        for(i=0; i < 16-ln; i++)
+        for(i=0; i < ASCII_LEN-ln; i++)
             printf("   ");
         if(ln < 8)
             printf(" ");
 
         printf(" %s\n\n", ascii_str);
     }
+    return;
 }
 
 /* Basic directory checks (stat() and whether the path is actually
@@ -97,8 +100,6 @@ is_valid_dir(const char *path)
 int
 verify_file_perms_ownership(const char *file)
 {
-    int res = 1;
-
 #if HAVE_STAT
     struct stat st;
 
@@ -115,10 +116,7 @@ verify_file_perms_ownership(const char *file)
                 "[-] file: %s is not a regular file or symbolic link.",
                 file
             );
-            /* when we start in enforcing this instead of just warning
-             * the user
-            res = 0;
-            */
+            return 0;
         }
 
         if((st.st_mode & (S_IRWXU|S_IRWXG|S_IRWXO)) != (S_IRUSR|S_IWUSR))
@@ -152,13 +150,106 @@ verify_file_perms_ownership(const char *file)
         {
             log_msg(LOG_ERR, "[-] stat() against file: %s returned: %s",
                 file, strerror(errno));
-            res = 0;
+            return 0;
         }
     }
 
 #endif
 
-    return res;
+    return 1;
+}
+
+static int
+add_argv(char **argv_new, int *argc_new,
+        const char *new_arg, const fko_srv_options_t * const opts)
+{
+    int buf_size = 0;
+
+    if(opts->verbose > 3)
+        log_msg(LOG_INFO, "[+] add_argv() + arg: %s", new_arg);
+
+    buf_size = strlen(new_arg) + 1;
+    argv_new[*argc_new] = calloc(1, buf_size);
+
+    if(argv_new[*argc_new] == NULL)
+    {
+        log_msg(LOG_INFO, "[*] Memory allocation error.");
+        return 0;
+    }
+    strlcpy(argv_new[*argc_new], new_arg, buf_size);
+
+    *argc_new += 1;
+
+    if(*argc_new >= MAX_CMDLINE_ARGS-1)
+    {
+        log_msg(LOG_ERR, "[*] max command line args exceeded.");
+        return 0;
+    }
+
+    argv_new[*argc_new] = NULL;
+
+    return 1;
+}
+
+int
+strtoargv(const char * const args_str, char **argv_new, int *argc_new,
+        const fko_srv_options_t * const opts)
+{
+    int       current_arg_ctr = 0, i;
+    char      arg_tmp[MAX_LINE_LEN] = {0};
+
+    for (i=0; i < (int)strlen(args_str); i++)
+    {
+        if (!isspace(args_str[i]))
+        {
+            arg_tmp[current_arg_ctr] = args_str[i];
+            current_arg_ctr++;
+        }
+        else
+        {
+            if(current_arg_ctr > 0)
+            {
+                arg_tmp[current_arg_ctr] = '\0';
+                if (add_argv(argv_new, argc_new, arg_tmp, opts) != 1)
+                {
+                    free_argv(argv_new, argc_new);
+                    return 0;
+                }
+                current_arg_ctr = 0;
+            }
+        }
+    }
+
+    /* pick up the last argument in the string
+    */
+    if(current_arg_ctr > 0)
+    {
+        arg_tmp[current_arg_ctr] = '\0';
+        if (add_argv(argv_new, argc_new, arg_tmp, opts) != 1)
+        {
+            free_argv(argv_new, argc_new);
+            return 0;
+        }
+    }
+    return 1;
+}
+
+void
+free_argv(char **argv_new, int *argc_new)
+{
+    int i;
+
+    if(argv_new == NULL || *argv_new == NULL)
+        return;
+
+    for (i=0; i < *argc_new; i++)
+    {
+        if(argv_new[i] == NULL)
+            break;
+        else
+            free(argv_new[i]);
+    }
+    return;
 }
 
 /***EOF***/
